@@ -17,8 +17,9 @@ using System.Globalization;
 using System.Xml;
 using System.Text;
 using System.Text.RegularExpressions;
+using System.Linq;
 
-#if !MOBILE_STATIC
+#if !FULL_AOT_DESKTOP && !MOBILE
 using Mono.Unix.Native;
 #endif
 
@@ -31,11 +32,14 @@ public class TestRunner
 	const string TEST_TIME_FORMAT = "mm\\:ss\\.fff";
 	const string ENV_TIMEOUT = "TEST_DRIVER_TIMEOUT_SEC";
 	const string MONO_PATH = "MONO_PATH";
+	const string MONO_GAC_PREFIX = "MONO_GAC_PREFIX";
 
 	class ProcessData {
 		public string test;
 		public StringBuilder stdout, stderr;
+		public object stdoutLock = new object (), stderrLock = new object ();
 		public string stdoutName, stderrName;
+		public TimeSpan duration;
 	}
 
 	class TestInfo {
@@ -50,16 +54,16 @@ public class TestRunner
 		bool verbose = false;
 		string testsuiteName = null;
 		string inputFile = null;
+		int repeat = 1;
 
 		string disabled_tests = null;
 		string runtime = "mono";
 		string config = null;
 		string mono_path = null;
+		string runtime_args = null;
+		string mono_gac_prefix = null;
 		var opt_sets = new List<string> ();
 
-		string aot_run_flags = null;
-		string aot_build_flags = null;
-
 		// Process options
 		int i = 0;
 		while (i < args.Length) {
@@ -95,6 +99,13 @@ public class TestRunner
 					}
 					runtime = args [i + 1];
 					i += 2;
+				} else if (args [i] == "--runtime-args") {
+					if (i + 1 >= args.Length) {
+						Console.WriteLine ("Missing argument to --runtime-args command line option.");
+						return 1;
+					}
+					runtime_args = (runtime_args ?? "") + " " + args [i + 1];
+					i += 2;
 				} else if (args [i] == "--config") {
 					if (i + 1 >= args.Length) {
 						Console.WriteLine ("Missing argument to --config command line option.");
@@ -131,13 +142,6 @@ public class TestRunner
 					}
 					inputFile = args [i + 1];
 					i += 2;
-				} else if (args [i] == "--runtime") {
-					if (i + 1 >= args.Length) {
-						Console.WriteLine ("Missing argument to --runtime command line option.");
-						return 1;
-					}
-					runtime = args [i + 1];
-					i += 2;
 				} else if (args [i] == "--mono-path") {
 					if (i + 1 >= args.Length) {
 						Console.WriteLine ("Missing argument to --mono-path command line option.");
@@ -146,23 +150,27 @@ public class TestRunner
 					mono_path = args [i + 1].Substring(0, args [i + 1].Length);
 
 					i += 2;
-				} else if (args [i] == "--aot-run-flags") {
+				} else if (args [i] == "--mono-gac-prefix") {
 					if (i + 1 >= args.Length) {
-						Console.WriteLine ("Missing argument to --aot-run-flags command line option.");
+						Console.WriteLine ("Missing argument to --mono-gac-prefix command line option.");
 						return 1;
 					}
-					aot_run_flags = args [i + 1].Substring(0, args [i + 1].Length);
-					i += 2;
-				} else if (args [i] == "--aot-build-flags") {
-					if (i + 1 >= args.Length) {
-						Console.WriteLine ("Missing argument to --aot-build-flags command line option.");
-						return 1;
-					}
-					aot_build_flags = args [i + 1].Substring(0, args [i + 1].Length);
+					mono_gac_prefix = args[i + 1];
 					i += 2;
 				} else if (args [i] == "--verbose") {
 					verbose = true;
 					i ++;
+				} else if (args [i] == "--repeat") {
+					if (i + 1 >= args.Length) {
+						Console.WriteLine ("Missing argument to --repeat command line option.");
+						return 1;
+					}
+					repeat = Int32.Parse (args [i + 1]);
+					if (repeat <= 1) {
+						Console.WriteLine ("Invalid argument to --repeat command line option, should be > 1");
+						return 1;
+					}
+					i += 2;
 				} else {
 					Console.WriteLine ("Unknown command line option: '" + args [i] + "'.");
 					return 1;
@@ -187,14 +195,30 @@ public class TestRunner
 		var tests = new List<string> ();
 
 		if (!String.IsNullOrEmpty (inputFile)) {
-			tests.AddRange (File.ReadAllLines (inputFile));
+			foreach (string l in File.ReadAllLines (inputFile)) {
+				for (int r = 0; r < repeat; ++r)
+					tests.Add (l);
+			}
 		} else {
 			// The remaining arguments are the tests
 			for (int j = i; j < args.Length; ++j)
-				if (!disabled.ContainsKey (args [j]))
-					tests.Add (args [j]);
+				if (!disabled.ContainsKey (args [j])) {
+					for (int r = 0; r < repeat; ++r)
+						tests.Add (args [j]);
+				}
 		}
 
+		if (!tests.Any ()) {
+			Console.WriteLine ("No tests selected, exiting.");
+			return 0;
+		}
+
+		/* If tests are repeated, we don't want the same test to run consecutively, so we need to randomise their order.
+		 * But to ease reproduction of certain order-based bugs (if and only if test A and B execute at the same time),
+		 * we want to use a constant seed so the tests always run in the same order. */
+		var random = new Random (0);
+		tests = tests.OrderBy (t => random.Next ()).ToList ();
+
 		var passed = new List<ProcessData> ();
 		var failed = new List<ProcessData> ();
 		var timedout = new List<ProcessData> ();
@@ -221,65 +245,6 @@ public class TestRunner
 				output_width = Math.Min (120, ti.test.Length);
 		}
 
-		if (aot_build_flags != null)  {
-			Console.WriteLine("AOT compiling tests");
-
-			object aot_monitor = new object ();
-			var aot_queue = new Queue<String> (tests); 
-
-			List<Thread> build_threads = new List<Thread> (concurrency);
-
-			for (int j = 0; j < concurrency; ++j) {
-				Thread thread = new Thread (() => {
-					while (true) {
-						String test_name;
-
-						lock (aot_monitor) {
-							if (aot_queue.Count == 0)
-								break;
-							test_name = aot_queue.Dequeue ();
-						}
-
-						string test_bitcode_output = test_name + "_bitcode_tmp";
-						string test_bitcode_arg = ",temp-path=" + test_bitcode_output;
-						string aot_args = aot_build_flags + test_bitcode_arg + " " + test_name;
-
-						Directory.CreateDirectory(test_bitcode_output);
-
-						ProcessStartInfo job = new ProcessStartInfo (runtime, aot_args);
-						job.UseShellExecute = false;
-						job.EnvironmentVariables[ENV_TIMEOUT] = timeout.ToString();
-						job.EnvironmentVariables[MONO_PATH] = mono_path;
-						Process compiler = new Process ();
-						compiler.StartInfo = job;
-
-						compiler.Start ();
-
-						if (!compiler.WaitForExit (timeout * 1000)) {
-							try {
-								compiler.Kill ();
-							} catch {
-							}
-							throw new Exception(String.Format("Timeout AOT compiling tests, output in {0}", test_bitcode_output));
-						} else if (compiler.ExitCode != 0) {
-							throw new Exception(String.Format("Error AOT compiling tests, output in {0}", test_bitcode_output));
-						} else {
-							Directory.Delete (test_bitcode_output, true);
-						}
-					}
-				});
-
-				thread.Start ();
-
-				build_threads.Add (thread);
-			}
-
-			for (int j = 0; j < build_threads.Count; ++j)
-				build_threads [j].Join ();
-
-			Console.WriteLine("Done compiling");
-		}
-
 		List<Thread> threads = new List<Thread> (concurrency);
 
 		DateTime test_start_time = DateTime.UtcNow;
@@ -306,19 +271,16 @@ public class TestRunner
 						Console.Write (".");
 					}
 
-					string test_invoke;
-
-					if (aot_run_flags != null)
-						test_invoke = aot_run_flags + " " + test;
-					else
-						test_invoke = test;
-
 					/* Spawn a new process */
-					string process_args;
-					if (opt_set == null)
-						process_args = test_invoke;
-					else
-						process_args = "-O=" + opt_set + " " + test_invoke;
+
+					string process_args = "";
+
+					if (opt_set != null)
+						process_args += " -O=" + opt_set;
+					if (runtime_args != null)
+						process_args += " " + runtime_args;
+
+					process_args += " " + test;
 
 					ProcessStartInfo info = new ProcessStartInfo (runtime, process_args);
 					info.UseShellExecute = false;
@@ -329,6 +291,8 @@ public class TestRunner
 						info.EnvironmentVariables["MONO_CONFIG"] = config;
 					if (mono_path != null)
 						info.EnvironmentVariables[MONO_PATH] = mono_path;
+					if (mono_gac_prefix != null)
+						info.EnvironmentVariables[MONO_GAC_PREFIX] = mono_gac_prefix;
 					Process p = new Process ();
 					p.StartInfo = info;
 
@@ -346,14 +310,16 @@ public class TestRunner
 					data.stderr = new StringBuilder ();
 
 					p.OutputDataReceived += delegate (object sender, DataReceivedEventArgs e) {
-						if (e.Data != null) {
-							data.stdout.AppendLine (e.Data);
+						lock (data.stdoutLock) {
+							if (e.Data != null)
+								data.stdout.AppendLine (e.Data);
 						}
 					};
 
 					p.ErrorDataReceived += delegate (object sender, DataReceivedEventArgs e) {
-						if (e.Data != null) {
-							data.stderr.AppendLine (e.Data);
+						lock (data.stderrLock) {
+							if (e.Data != null)
+								data.stderr.AppendLine (e.Data);
 						}
 					};
 
@@ -365,18 +331,15 @@ public class TestRunner
 					p.BeginErrorReadLine ();
 
 					if (!p.WaitForExit (timeout * 1000)) {
+						var end = DateTime.UtcNow;
+						data.duration =  end - start;
+
 						lock (monitor) {
 							timedout.Add (data);
 						}
 
-#if !MOBILE_STATIC
 						// Force the process to print a thread dump
-						try {
-							Syscall.kill (p.Id, Signum.SIGQUIT);
-							Thread.Sleep (1000);
-						} catch {
-						}
-#endif
+						TryThreadDump (p.Id, data);
 
 						if (verbose) {
 							output.Write ($"timed out ({timeout}s)");
@@ -388,22 +351,24 @@ public class TestRunner
 						}
 					} else if (p.ExitCode != expectedExitCode) {
 						var end = DateTime.UtcNow;
+						data.duration =  end - start;
 
 						lock (monitor) {
 							failed.Add (data);
 						}
 
 						if (verbose)
-							output.Write ("failed, time: {0}, exit code: {1}", (end - start).ToString (TEST_TIME_FORMAT), p.ExitCode);
+							output.Write ("failed, time: {0}, exit code: {1}", data.duration.ToString (TEST_TIME_FORMAT), p.ExitCode);
 					} else {
 						var end = DateTime.UtcNow;
+						data.duration =  end - start;
 
 						lock (monitor) {
 							passed.Add (data);
 						}
 
 						if (verbose)
-							output.Write ("passed, time: {0}", (end - start).ToString (TEST_TIME_FORMAT));
+							output.Write ("passed, time: {0}", data.duration.ToString (TEST_TIME_FORMAT));
 					}
 
 					p.Close ();
@@ -467,7 +432,7 @@ public class TestRunner
 			writer.WriteStartElement ("test-suite");
 			writer.WriteAttributeString ("name", String.Format ("{0}-tests.dummy", testsuiteName));
 			writer.WriteAttributeString ("success", (nfailed + ntimedout == 0).ToString());
-			writer.WriteAttributeString ("time", test_time.Seconds.ToString());
+			writer.WriteAttributeString ("time", test_time.TotalSeconds.ToString(CultureInfo.InvariantCulture));
 			writer.WriteAttributeString ("asserts", (nfailed + ntimedout).ToString());
 			//     <results>
 			writer.WriteStartElement ("results");
@@ -475,7 +440,7 @@ public class TestRunner
 			writer.WriteStartElement ("test-suite");
 			writer.WriteAttributeString ("name","MonoTests");
 			writer.WriteAttributeString ("success", (nfailed + ntimedout == 0).ToString());
-			writer.WriteAttributeString ("time", test_time.Seconds.ToString());
+			writer.WriteAttributeString ("time", test_time.TotalSeconds.ToString(CultureInfo.InvariantCulture));
 			writer.WriteAttributeString ("asserts", (nfailed + ntimedout).ToString());
 			//         <results>
 			writer.WriteStartElement ("results");
@@ -483,7 +448,7 @@ public class TestRunner
 			writer.WriteStartElement ("test-suite");
 			writer.WriteAttributeString ("name", testsuiteName);
 			writer.WriteAttributeString ("success", (nfailed + ntimedout == 0).ToString());
-			writer.WriteAttributeString ("time", test_time.Seconds.ToString());
+			writer.WriteAttributeString ("time", test_time.TotalSeconds.ToString(CultureInfo.InvariantCulture));
 			writer.WriteAttributeString ("asserts", (nfailed + ntimedout).ToString());
 			//             <results>
 			writer.WriteStartElement ("results");
@@ -494,7 +459,7 @@ public class TestRunner
 				writer.WriteAttributeString ("name", String.Format ("MonoTests.{0}.{1}", testsuiteName, pd.test));
 				writer.WriteAttributeString ("executed", "True");
 				writer.WriteAttributeString ("success", "True");
-				writer.WriteAttributeString ("time", "0");
+				writer.WriteAttributeString ("time", pd.duration.TotalSeconds.ToString(CultureInfo.InvariantCulture));
 				writer.WriteAttributeString ("asserts", "0");
 				writer.WriteEndElement ();
 			}
@@ -505,7 +470,7 @@ public class TestRunner
 				writer.WriteAttributeString ("name", String.Format ("MonoTests.{0}.{1}", testsuiteName, pd.test));
 				writer.WriteAttributeString ("executed", "True");
 				writer.WriteAttributeString ("success", "False");
-				writer.WriteAttributeString ("time", "0");
+				writer.WriteAttributeString ("time", pd.duration.TotalSeconds.ToString(CultureInfo.InvariantCulture));
 				writer.WriteAttributeString ("asserts", "1");
 				writer.WriteStartElement ("failure");
 				writer.WriteStartElement ("message");
@@ -524,7 +489,7 @@ public class TestRunner
 				writer.WriteAttributeString ("name", String.Format ("MonoTests.{0}.{1}_timedout", testsuiteName, pd.test));
 				writer.WriteAttributeString ("executed", "True");
 				writer.WriteAttributeString ("success", "False");
-				writer.WriteAttributeString ("time", "0");
+				writer.WriteAttributeString ("time", pd.duration.TotalSeconds.ToString(CultureInfo.InvariantCulture));
 				writer.WriteAttributeString ("asserts", "1");
 				writer.WriteStartElement ("failure");
 				writer.WriteStartElement ("message");
@@ -611,4 +576,119 @@ public class TestRunner
 		// Char ::= #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000-#x10FFFF]	/* any Unicode character, excluding the surrogate blocks, FFFE, and FFFF. */
 		return Regex.Replace (text, @"[^\x09\x0A\x0D\x20-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]", "");
 	}
+
+	static void TryThreadDump (int pid, ProcessData data)
+	{
+		try {
+			TryGDB (pid, data);
+			return;
+		} catch {
+		}
+
+#if !FULL_AOT_DESKTOP && !MOBILE
+		/* LLDB cannot produce managed stacktraces for all the threads */
+		try {
+			Syscall.kill (pid, Signum.SIGQUIT);
+			Thread.Sleep (1000);
+		} catch {
+		}
+#endif
+
+		try {
+			TryLLDB (pid, data);
+			return;
+		} catch {
+		}
+	}
+
+	static void TryLLDB (int pid, ProcessData data)
+	{
+		string filename = Path.GetTempFileName ();
+
+		using (StreamWriter sw = new StreamWriter (new FileStream (filename, FileMode.Open, FileAccess.Write)))
+		{
+			sw.WriteLine ("process attach --pid " + pid);
+			sw.WriteLine ("thread list");
+			sw.WriteLine ("thread backtrace all");
+			sw.WriteLine ("detach");
+			sw.WriteLine ("quit");
+			sw.Flush ();
+
+			ProcessStartInfo psi = new ProcessStartInfo {
+				FileName = "lldb",
+				Arguments = "--batch --source \"" + filename + "\" --no-lldbinit",
+				UseShellExecute = false,
+				RedirectStandardError = true,
+				RedirectStandardOutput = true,
+			};
+
+			using (Process process = new Process { StartInfo = psi })
+			{
+				process.OutputDataReceived += delegate (object sender, DataReceivedEventArgs e) {
+					lock (data.stdoutLock) {
+						if (e.Data != null)
+							data.stdout.AppendLine (e.Data);
+					}
+				};
+
+				process.ErrorDataReceived += delegate (object sender, DataReceivedEventArgs e) {
+					lock (data.stderrLock) {
+						if (e.Data != null)
+							data.stderr.AppendLine (e.Data);
+					}
+				};
+
+				process.Start ();
+				process.BeginOutputReadLine ();
+				process.BeginErrorReadLine ();
+				if (!process.WaitForExit (60 * 1000))
+					process.Kill ();
+			}
+		}
+	}
+
+	static void TryGDB (int pid, ProcessData data)
+	{
+		string filename = Path.GetTempFileName ();
+
+		using (StreamWriter sw = new StreamWriter (new FileStream (filename, FileMode.Open, FileAccess.Write)))
+		{
+			sw.WriteLine ("attach " + pid);
+			sw.WriteLine ("info threads");
+			sw.WriteLine ("thread apply all p mono_print_thread_dump(0)");
+			sw.WriteLine ("thread apply all backtrace");
+			sw.Flush ();
+
+			ProcessStartInfo psi = new ProcessStartInfo {
+				FileName = "gdb",
+				Arguments = "-batch -x \"" + filename + "\" -nx",
+				UseShellExecute = false,
+				RedirectStandardError = true,
+				RedirectStandardOutput = true,
+			};
+
+			using (Process process = new Process { StartInfo = psi })
+			{
+				process.OutputDataReceived += delegate (object sender, DataReceivedEventArgs e) {
+					lock (data.stdoutLock) {
+						if (e.Data != null)
+							data.stdout.AppendLine (e.Data);
+					}
+				};
+
+				process.ErrorDataReceived += delegate (object sender, DataReceivedEventArgs e) {
+					lock (data.stderrLock) {
+						if (e.Data != null)
+							data.stderr.AppendLine (e.Data);
+					}
+				};
+
+				process.Start ();
+				process.BeginOutputReadLine ();
+				process.BeginErrorReadLine ();
+				if (!process.WaitForExit (60 * 1000))
+					process.Kill ();
+			}
+		}
+	}
 }
